"""
New Zealand Business Number
Classification Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from typing import List, Dict, TypeVar, Type, Optional

Self = TypeVar('Self', bound='Classification')


class Classification:

    def __init__(
        self,
        unique_identifier: Optional[str],
        classification_code: str,
        classification_description: str
    ) -> None:

        self._unique_identifier = unique_identifier
        self._classification_code = classification_code
        self._classification_description = classification_description

        return

    unique_identifier: Optional[str] = property(
        lambda s: s._unique_identifier
    )
    classification_code = property(lambda s: s._classification_code)
    classification_description = property(
        lambda s: s._classification_description
    )

    @classmethod
    def decode(
        Self: Type[Self],
        data: Dict[str, str]
    ) -> Self:

        return Classification(
            unique_identifier=(data['uniqueIdentifier']),
            classification_code=data['classificationCode'],
            classification_description=data['classificationDescription']
        )

    @classmethod
    def decode_many(
        Self: Type[Self],
        data: List[Dict[str, str]]
    ) -> List[Self]:

        return [Self.decode(c) for c in data]



"""
New Zealand Business Number
Order Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from enum import Enum


class OrderBy(Enum):
    NZBN = 'NZBN'
    ENTITY_NAME = 'ENTITY_NAME'
    ENTITY_STATUS = 'ENTITY_STATUS'
    ENTITY_TYPE = 'ENTITY_TYPE_CODE'


class Order(Enum):
    ASCENDING = 'ASC'
    DESCENDING = 'DESC'



"""
New Zealand Business Number
Entity Type Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from enum import Enum


class EntityType(Enum):

    NZ_COMPANY = 'NZCompany'
    OVERSEAS_COMPANY = 'OverseasCompany'
    SOLE_TRADER = 'SoleTrader'
    PARTNERSHIP = 'Partnership'
    TRUST = 'Trust'
    BUILDING_SOCIETY = 'BuildingSociety'
    CHARITABLE_TRUST = 'CharitableTrust'
    CREDIT_UNION = 'CreditUnion'
    FRIENDLY_SOCIETY = 'FriendlySociety'
    INCORPORATED_SOCIETY = 'IncorporatedSociety'
    INDUSTRIAL_PROVIDENT_SOCIETY = 'IndustrialAndProvidentSociety'
    LIMITED_PARTNERSHIP_NZ = 'LimitedPartnershipNz'
    LIMITED_PARTNERSHIP_OVERSEAS = 'LimitedPartnershipOverseas'
    SPECIAL_BODIES = 'SpecialBodies'
    SPECIAL_BODY = 'SpecialBody'
    TRADING_TRUST = 'Trading_Trust'
    SOLE_TRADER_ALTERNATE = 'Sole_Trader'
    B = 'B'
    I = 'I'
    D = 'D'
    F = 'F'
    N = 'N'
    S = 'S'
    T = 'T'
    Y = 'Y'
    Z = 'Z'
    GOVT_CENTRAL = 'GovtCentral'
    GOVT_EDU = 'GovtEdu'
    GOVT_LOCAL = 'GovtLocal'
    GOVT_OTHER = 'GovtOther'
    G = 'G'
    LTD = 'LTD'
    ULTD = 'ULTD'
    COOP = 'COOP'
    ASIC = 'ASIC'
    NON_ASIC = 'NON_ASIC'



"""
New Zealand Business Number
Entity Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from typing import List, Optional, Dict, Any
from nzbn.address import Address, AddressType
from nzbn.disposition import Disposition
from nzbn.entity_type import EntityType
from nzbn.entity_status import EntityStatus
from nzbn.errors.error import NzbnError
from nzbn.errors.type_error import NzbnTypeError
from nzbn.http.api_request import ApiRequest
from nzbn.http.method import HTTPMethod
from nzbn.http.query_parameter import QueryParameter
from nzbn.http.query_parameters import QueryParameters
from nzbn.time import NzbnTime
from nzbn.trading_name import TradingName
from nzbn.classification import Classification
from typing import TypeVar, Type
from nzbn.address import Address

Self = TypeVar('Self', bound='Entity')


class Entity:

    path = '/entities'

    def __init__(
        self,
        entity_name: str,
        status: EntityStatus,
        nzbn: str,
        entity_type: EntityType,
        trading_names: List[TradingName],
        classifications: List[Classification],
        registration_date: NzbnTime,
        disposition: Disposition,
        addresses: List[Address]
    ) -> None:

        self._entity_name = entity_name
        self._status = status
        self._nzbn = nzbn
        self._entity_type = entity_type
        self._trading_names = trading_names
        self._classifications = classifications
        self._registration_date = registration_date
        self._disposition = disposition
        self._addresses = addresses

        return

    entity_name: str = property(lambda s: s._entity_name)
    status: EntityStatus = property(lambda s: s._status)
    nzbn: str = property(lambda s: s._nzbn)
    entity_type: EntityType = property(lambda s: s._entity_type)
    trading_names: List[str] = property(lambda s: s._trading_names)
    classifications: List[Classification] = property(
        lambda s: s._classifications
    )
    registration_date: NzbnTime = property(lambda s: s._registration_date)
    disposition: Disposition = property(
        lambda s: s._disposition
    )
    addresses: List[Address] = property(lambda s: s._addresses)

    registered_address: Optional[Address] = property(
        lambda s: s._parse_registered_address()
    )

    def _parse_registered_address(self) -> Optional[Address]:

        registered = [
            a for a in self._addresses if (
                a.not_expired and a.address_type == AddressType.REGISTERED
            )         
        ]

        if len(registered) < 1:
            return None
        
        return registered[0]

    @classmethod
    def decode(
        Self: Type[Self],
        data: Dict[str, Any],
        disposition: Optional[Disposition] = None
    ) -> Self:

        return Self(
            entity_name=data['entityName'],
            status=EntityStatus(data['entityStatusCode']),
            nzbn=data['nzbn'],
            entity_type=EntityType(data['entityTypeCode']),
            trading_names=TradingName.decode_many(data['tradingNames']),
            classifications=Classification.decode_many(
                data['classifications']
            ) if 'classifications' in data else [],
            registration_date=NzbnTime.decode(data['registrationDate']),
            disposition=disposition or Disposition(
                1,
                1,
                1,
                1
            ),
            addresses=Address.decode_many(data['addresses']['addressList'])
        )

    @classmethod
    def retrieve(
        Self: Type[Self],
        access_token: str,
        nzbn: str,
        sandbox: bool = False
    ) -> Optional[Self]:

        if not isinstance(access_token, str):
            raise NzbnTypeError('access_token', access_token, 'str')
        
        if not isinstance(nzbn, str):
            raise NzbnTypeError('nzbn', nzbn, 'str')
        
        if len(nzbn) != 13:
            raise NzbnError('nzbn must be 13 characters')
        
        for character in nzbn:
            if character not in '0123456789':
                raise NzbnError('nzbn must be only numeric characters')
            continue

        result = ApiRequest.make(
            path=Self.path + '/' + nzbn,
            method=HTTPMethod.GET,
            access_token=access_token,
            sandbox=sandbox
        )
        
        if result is None:
            return None
        
        return Self.decode(result)



"""
New Zealand Business Number
API Error Class
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from nzbn.errors.error import NzbnError
from typing import Optional


class NzbnApiError(NzbnError):

    def __init__(
        self,
        code: int,
        body: Optional[str]
    ) -> None:

        self._code = code
        self._body = body

        super().__init__('NZBN API emitted error with HTTP code {c}. {b}\
'.format(
            c=str(code),
            b=(
                'Examine .body property for more information.' if (
                    self._body is not None
                ) else 'No further information provided by NZBN API.'
            )
        ))

        return
    
    body = property(lambda s: s._body)



"""
New Zealand Business Number
Disposition Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""


class Disposition:

    def __init__(
        self,
        sequence: int,
        offset: int,
        count: int,
        limit: int
    ) -> None:

        self._sequence = sequence
        self._offset = offset
        self._count = count
        self._limit = limit

        return



"""
New Zealand Business Number
Query Parameters Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from nzbn.http.query_parameter import QueryParameter
from typing import List


class QueryParameters:
    """A collection of URL Query path parameters"""

    def __init__(self, targets: List[QueryParameter]) -> None:

        assert isinstance(targets, list)
        assert False not in [isinstance(t, QueryParameter) for t in targets]

        self._targets = targets
        return

    query_string = property(lambda s: s._form_query_string())

    def _form_query_string(self) -> str:
        if len(self._targets) < 1:
            return ''

        query = '?' + str(self._targets[0])

        for target in self._targets[1:]:
            query += '&' + str(target)

        return query

    def add_to(self, url: str) -> str:
        """Return a URL with parameters bolted on"""
        if len(self._targets) < 1:
            return url

        return url + self.query_string



"""
New Zealand Business Number
Trading Name Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from typing import Type,  TypeVar, Dict, List, Optional
from nzbn.time import NzbnTime

Self = TypeVar('Self', bound='TradingName')


class TradingName:

    def __init__(
        self,
        unique_identifier: str,
        name: str,
        start_date: Optional[NzbnTime],
        end_date: Optional[NzbnTime]
    ) -> None:

        self._unique_identifier = unique_identifier
        self._name = name
        self._start_date = start_date
        self._end_date = end_date

        return
    
    @classmethod
    def decode(Self: Type[Self], data: Dict[str, str]) -> Self:
        return Self(
            unique_identifier=data['uniqueIdentifier'],
            name=data['name'],
            start_date=NzbnTime.optionally_decode(data['startDate']),
            end_date=NzbnTime.optionally_decode(data['endDate'])
        )

    @classmethod
    def decode_many(
        Self: Type[Self],
        data: List[Dict[str, str]]
    ) -> List[Self]:

        return [Self.decode(t) for t in data]



"""
New Zealand Business Number
Api Request Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
import json
from nzbn.http.method import HTTPMethod
from typing import Optional, Dict, Union, List
from nzbn.http.query_parameters import QueryParameters
from nzbn.errors.api import NzbnApiError
from urllib.request import HTTPError
from urllib.request import Request
from urllib.request import urlopen
from nzbn.version import VERSION as AGENT_VERSION

SANDBOX_ENDPOINT = 'https://sandbox.api.business.govt.nz/services/v4/nzbn'
API_ENDPOINT = 'https://api.business.govt.nz/services/v4/nzbn'
USER_AGENT = 'NZBN Python ' + AGENT_VERSION


class ApiRequest:

    @staticmethod
    def make(
        path: str,
        method: HTTPMethod,
        access_token: Optional[str] = None,
        data: Optional[Union[Dict, List[Dict]]] = None,
        query_parameters: Optional[QueryParameters] = None,
        api_endpoint: Optional[str] = None,
        throw_on_404: bool = False,
        sandbox: bool = False,
        timeout: int = 10
    ) -> Optional[Union[Dict, List[Dict]]]:
        """Return the decoded json body of a response from the Procuret API"""

        api_endpoint = api_endpoint or (
            SANDBOX_ENDPOINT if sandbox is True
            else API_ENDPOINT
        )

        url = api_endpoint + path
        if query_parameters is not None:
            url = query_parameters.add_to(url)

        headers = {
            'User-Agent': USER_AGENT,
            'Authorization': 'Bearer ' + access_token
        }

        encoded_data: Optional[bytes] = None
        if data is not None:
            encoded_data = json.dumps(data).encode('utf-8')
            headers['Content-Type'] = 'application/json'

        request = Request(
            url=url,
            method=method.value,
            data=encoded_data,
            headers=headers
        )

        try:
            response = urlopen(request, timeout=timeout).read()
        except HTTPError as error:
            if error.code == 404 and throw_on_404 is False:
                return None
            body: Optional[str] = None
            try:
                body = error.read()
                try:
                    jsonbody = json.loads(body)
                    if 'errorDescription' in jsonbody and (
                        'Object Not Found' in jsonbody['errorDescription']
                    ) and throw_on_404 is False:
                        return None
                except Exception:
                    pass
            except Exception:
                pass
            raise NzbnApiError(error.code, body=body)

        return json.loads(response)



from nzbn.entity import Entity
from nzbn.abbreviated_entity import AbbreviatedEntity
from nzbn.classification import Classification
from nzbn.disposition import Disposition
from nzbn.entity_status import EntityStatus
from nzbn.entity_type import EntityType
from nzbn.trading_name import TradingName
from nzbn.time import NzbnTime
from nzbn.address import Address, AddressType

from nzbn.errors.error import NzbnError
from nzbn.errors.api import NzbnApiError
from nzbn.errors.type_error import NzbnTypeError



"""
New Zealand Business Number
Time Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from datetime import datetime, tzinfo, timedelta
from typing import Optional, TypeVar, Type

Self = TypeVar('Self', bound='NzbnTime')


class TimeZoneUTC(tzinfo):
    """
    UTC timezone.
    """
    def utcoffset(self, dt):
        return timedelta(0)

    def tzname(self, dt):
        return 'UTC'

    def dst(self, dt):
        return timedelta(0)


UTC = TimeZoneUTC()


class NzbnTime(datetime):

    _FORMAT = '%Y-%m-%dT%H:%M:%S.%f%z'

    @classmethod
    def decode(Self: Type[Self], data: str) -> Self:

        time = datetime.strptime(data, Self._FORMAT)
        time = time - time.tzinfo.utcoffset(time)

        return NzbnTime(
            year=time.year,
            month=time.month,
            day=time.day,
            hour=time.hour,
            minute=time.minute,
            second=time.second,
            microsecond=time.microsecond,
            tzinfo=UTC
        )

    @classmethod
    def optionally_decode(
        Self: Type[Self],
        data: Optional[str]
    ) -> Optional[Self]:

        if data is None:
            return None
        
        return Self.decode(data)

    @classmethod
    def utcnow(cls: Type[Self]) -> Self:
        time = datetime.utcnow()
        return cls._from_datetime(time)

    @classmethod
    def _from_datetime(cls: Type[Self], time: datetime) -> Self:
        return cls(
            year=time.year,
            month=time.month,
            day=time.day,
            hour=time.hour,
            minute=time.minute,
            second=time.second,
            microsecond=time.microsecond,
            tzinfo=UTC
        )



"""
New Zealand Business Number
HTTP Method Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from enum import Enum


class HTTPMethod(Enum):
    GET = 'GET'
    POST = 'POST'
    DELETE = 'DELETE'
    PUT = 'PUT'
    PATCH = 'PATCH'



"""
New Zealand Business Number
Address Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from datetime import datetime
from nzbn.time import NzbnTime
from enum import Enum
from typing import Optional, Type, TypeVar, Dict, List

Self = TypeVar('Self', bound='Address')


class AddressType(Enum):

    POSTAL = 'POSTAL'
    REGISTERED = 'REGISTERED'
    SERVICE = 'SERVICE'
    RECORDS = 'RECORDS'
    SHAREREGISTER = 'SHAREREGISTER'
    OFFICE = 'OFFICE'
    DELIVERY = 'DELIVERY'
    INVOICE = 'INVOICE'


class Address:

    def __init__(
        self,
        unique_identifier: str,
        start_date: Optional[NzbnTime],
        end_date: Optional[NzbnTime],
        care_of: Optional[str],
        address_1: str,
        address_2: Optional[str],
        address_3: Optional[str],
        address_4: Optional[str],
        post_code: Optional[str],
        country_code: Optional[str],
        address_type: Optional[AddressType],
        paf_id: Optional[str]
    ) -> None:

        self._unique_identifier = unique_identifier
        self._start_date = start_date
        self._end_date = end_date
        self._care_of = care_of
        self._address_1 = address_1
        self._address_2 = address_2
        self._address_3 = address_3
        self._address_4 = address_4
        self._post_code = post_code
        self._country_code = country_code
        self._address_type = address_type
        self._paf_id = paf_id

        return

    unique_identifier = property(lambda s: s._unqiue_identifier)
    start_date = property(lambda s: s._start_date)
    end_date = property(lambda s: s._end_date)

    care_of: Optional[str] = property(lambda s: s._care_of)
    line1: str = property(lambda s: s._address_1)
    line2: Optional[str] = property(lambda s: s._address_2)
    line3: Optional[str] = property(lambda s: s._address_3)
    line4: Optional[str] = property(lambda s: s._address_4)

    post_code: Optional[str] = property(lambda s: s._post_code)
    country_code: Optional[str] = property(lambda s: s._country_code)

    address_type: Optional[AddressType] = property(
        lambda s: s._address_type
    )
    paf_id: Optional[str] = property(lambda s: s._paf_id)

    expired: bool = property(
        lambda s: (
            s._end_date is not None and s._end_date <= datetime.utcnow()
        )
    )
    not_expired: bool = property(lambda s: not s.expired)

    @classmethod
    def decode(
        Self: Type[Self],
        data: Dict[str, Optional[str]]
    ) -> Self:

        return Address(
            unique_identifier=data['uniqueIdentifier'],
            start_date=NzbnTime.optionally_decode(data['startDate']),
            end_date=NzbnTime.optionally_decode(data['endDate']),
            care_of=data['careOf'],
            address_1=data['address1'],
            address_2=data['address2'],
            address_3=data['address3'],
            address_4=data['address4'],
            post_code=data['postCode'],
            country_code=data['countryCode'],
            address_type=(
                AddressType(data['addressType']) if data['addressType']
                is not None else None
            ),
            paf_id=data['pafId']
        )        

    @classmethod
    def decode_many(
        Self: Type[Self],
        data: List[Dict[str, Optional[str]]]
    ) -> List[Self]:

        return [Self.decode(a) for a in data]



"""
New Zealand Business Number
Abbreviated Entity Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from typing import List, Optional, Dict, Union, Any
from nzbn.disposition import Disposition
from nzbn.entity_type import EntityType
from nzbn.entity_status import EntityStatus
from nzbn.errors.error import NzbnError
from nzbn.errors.type_error import NzbnTypeError
from nzbn.http.api_request import ApiRequest
from nzbn.http.method import HTTPMethod
from nzbn.http.query_parameter import QueryParameter
from nzbn.http.query_parameters import QueryParameters
from nzbn.time import NzbnTime
from nzbn.trading_name import TradingName
from nzbn.classification import Classification
from typing import TypeVar, Type

Self = TypeVar('Self', bound='AbbreviatedEntity')


class AbbreviatedEntity:

    path = '/entities'

    def __init__(
        self,
        entity_name: str,
        status: EntityStatus,
        nzbn: str,
        entity_type: EntityType,
        trading_names: List[TradingName],
        classifications: List[Classification],
        registration_date: Optional[NzbnTime],
        disposition: Disposition
    ) -> None:

        self._entity_name = entity_name
        self._status = status
        self._nzbn = nzbn
        self._entity_type = entity_type
        self._trading_names = trading_names
        self._classifications = classifications
        self._registration_date = registration_date
        self._disposition = disposition

        return

    entity_name: str = property(lambda s: s._entity_name)
    status: EntityStatus = property(lambda s: s._status)
    nzbn: str = property(lambda s: s._nzbn)
    entity_type: EntityType = property(lambda s: s._entity_type)
    trading_names: List[str] = property(lambda s: s._trading_names)
    classifications: List[Classification] = property(
        lambda s: s._classifications
    )
    registration_date: Optional[NzbnTime] = property(
        lambda s: s._registration_date
    )
    disposition: Disposition = property(
        lambda s: s._disposition
    )

    @classmethod
    def decode_encapsulated_list(
        Self: Type[Self],
        data: Dict[str, Any]
    ) -> List[Self]:

        total = data['totalItems']
        page = data['page']
        page_size = data['pageSize']

        offset = (page - 1) * page_size

        sequence = offset

        entities: List[AbbreviatedEntity] = []

        for entity_data in data['items']:
            sequence += 1
            disposition = Disposition(
                sequence=sequence,
                offset=offset,
                count=total,
                limit=page_size
            )
            entities.append(AbbreviatedEntity.decode(
                data=entity_data,
                disposition=disposition
            ))
            continue
        
        return entities

    @classmethod
    def decode(
        Self: Type[Self],
        data: Dict[str, Any],
        disposition: Optional[Disposition] = None
    ) -> Self:

        return Self(
            entity_name=data['entityName'],
            status=EntityStatus(data['entityStatusCode']),
            nzbn=data['nzbn'],
            entity_type=EntityType(data['entityTypeCode']),
            trading_names=TradingName.decode_many(data['tradingNames']),
            classifications=Classification.decode_many(
                data['classifications']
            ) if 'classifications' in data else [],
            registration_date=NzbnTime.optionally_decode(
                data['registrationDate']
            ),
            disposition=disposition or Disposition(
                1,
                1,
                1,
                1
            )
        )

    @classmethod
    def retrieve_many(
        Self: Type[Self],
        access_token: str,
        page: int = 0,
        limit: int = 20,
        search_text: Optional[str] = None,
        entity_status: Optional[Union[
            EntityStatus,
            List[EntityStatus]
        ]] = None,
        entity_type: Optional[Union[
            EntityType,
            List[EntityType]
        ]] = None,
        sandbox: bool = False
    ) -> List[Self]:


        if not isinstance(page, int):
            raise NzbnTypeError('offset', page, 'int')

        if not isinstance(limit, int):
            raise NzbnTypeError('limit', limit, 'int')

        if page < 0:
            raise NzbnError('page must be >= 0')

        if limit < 0:
            raise NzbnError('limit must be >= 0')

        parameters: List[QueryParameter] = [
            QueryParameter(key='page', value=page),
            QueryParameter(key='page-size', value=limit)
        ]

        if search_text is not None:
            if not isinstance(search_text, str):
                raise NzbnTypeError(
                    'search_text',
                    search_text,
                    'Optional[str]'
                )
            parameters.append(QueryParameter(
                key='search-term',
                value=search_text
            ))
            pass

        if entity_status is not None:

            def raise_error() -> None:
                raise NzbnTypeError(
                    'entity_status',
                    entity_status,
                    'Optional[Union[EntityStatus, List[EntityStatus]]]'
                )

            if isinstance(entity_status, list):
                if False in [
                    isinstance(e, EntityStatus) for e in entity_status
                ]:
                    raise_error()
                pass
            else:
                if not isinstance(entity_status, EntityStatus):
                    raise_error()
    
            parameters.append(QueryParameters(
                key='entity-status',
                value=(
                    ','.join([e.value for e in entity_status] if (
                        isinstance(entity_status, list)
                    ) else entity_status.value
                )
            )))

            pass

        if entity_type is not None:

            def raise_error() -> None:
                raise NzbnTypeError(
                    'entity_type',
                    entity_status,
                    'Optional[Union[EntityType, List[EntityType]]]'
                )

            if isinstance(entity_type, list):
                if False in [
                    isinstance(e, EntityType) for e in entity_type
                ]:
                    raise_error()
                pass
            else:
                if not isinstance(entity_type, EntityType):
                    raise_error()
    
            parameters.append(QueryParameters(
                key='entity-type',
                value=(
                    ','.join([e.value for e in entity_type] if (
                        isinstance(entity_type, list)
                    ) else entity_type.value
                )
            )))

            pass

        result = ApiRequest.make(
            path=Self.path,
            method=HTTPMethod.GET,
            query_parameters=QueryParameters(parameters),
            access_token=access_token,
            sandbox=sandbox
        )

        return Self.decode_encapsulated_list(result)



"""
New Zealand Business Number
PyPI Setup Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from setuptools import setup, find_packages
from os import path
from codecs import open
from nzbn.version import VERSION

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'readme.md'), encoding='utf-8') as readme_file:
    LONG_DESCRIPTION = readme_file.read()

setup(
    name='nzbn',
    version=VERSION,
    description='New Zealand Business Number library',
    long_description=LONG_DESCRIPTION,
    url='https://github.com/procuret/nzbn-python',
    author='Procuret',
    author_email='hugh@procuret.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries',
        'Topic :: Office/Business :: Financial'
    ],
    keywords='library http api web payments finance nzbn new-zealand nz busine\
ss business-number',
    packages=find_packages(),
    long_description_content_type="text/markdown",
    python_requires='>=3.6',
    project_urls={
        'Github Repository': 'https://github.com/procuret/nzbn-python',
        'About': 'https://github.com/procuret/nzbn-python'
    }
)



"""
New Zealand Business Number
Entity Status Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from enum import Enum


class EntityStatus(Enum):

    REGISTERED = '50'
    VOLUNTARY_ADMINISTRATION = '55'
    VOLUNTARY_ADMINISTRATION_AND_RECEIVERSHIP = '56'
    RECEIVERSHIP_AND_VOLUNTARY_ADMINISTRATION = '57'
    IN_LIQUIDATION = '60'
    IN_RECEIVERSHIP_AND_LIQUIDATION = '61'
    REMOVED_AND_IN_LIQUIDATION = '62'
    IN_RECEIVERSHIP_LIQUIDIATION_AND_VOLUNTARY_ADMINISTRATION = '63'
    IN_LIQUIDATION_AND_RECEIVERSHIP_AND_VOLUNTARY_ADMINISTRATION = '64'
    VOLUNTARY_ADMINISTRATION_AND_LIQUIDATION = '65'
    IN_LIQUIDATION_AND_VOLUNTARY_ADMINISTRATION_AND_RECEIVERSHIP = '66'
    IN_RECEIVERSHIP = '70'
    IN_RECEIVERSHIP_AND_LIQUIDATION_DUPLICATE = '71'
    IN_STATUTORY_ADMINISTRATION = '72'
    REMOVED = '80'
    INACTIVE = '90'
    REMOVED_CLOSED = '91'



"""
New Zealand Business Number
Error Module
author: hugh@procuret.com
© Procuret Operating Pty Ltd
"""
from typing import Any
from nzbn.errors.error import NzbnError


class NzbnTypeError(NzbnError):

    def __init__(
        self,
        value_name: str,
        provided_value: Any,
        expected_type_name: str
    ) -> None:
    
        description = 'Expected type `{t}` for value `{v}`, got `{p}`'.format(
            t=expected_type_name,
            p=str(type(provided_value)),
            v=value_name
        )

        return super().__init__(description)
